import random
from typing import cast, Any, Dict, List, MutableSequence, Sequence, TypeVar

T = TypeVar("T")


overrides: Dict[str,Any] = {}


def randomly(key: str, odds: int) -> bool:
    """Overrideable random func which returns true at 1:ODDS odds."""
    if key in overrides:
        return cast(bool, overrides[key])
    ans = random.randint(0, odds-1) < 1
    try:
        log.debug(f"Rand: {key}: {odds} A: {ans}")
    except Exception:
        pass
    return ans


def randint(key: str, bottom: int, top: int) -> int:
    """Overrideable random func which returns an integer bottom <= i <= top."""
    if key in overrides:
        return cast(int, overrides[key])
    ans = random.randint(int(bottom), int(top))
    try: 
        # Very Very Verbose...
        if key not in [
                'move_player_x',
                'move_player_y',
                'hog_trigger',
                'team_battle_trigger',
                'calamity_trigger',
                'godsend_trigger',
                'evilness_trigger',
                'goodness_trigger'
                ]:
            log.debug(f"Rand: {key}: {int(bottom)} to {int(top)} A: {ans}")
    except Exception:
        pass
    return ans


def gauss(key: str, mu: float, sigma: float) -> int:
    """Overrideable func which returns an random int with gaussian distribution."""
    if key in overrides:
        return cast(int, overrides[key])
    ans = int(random.gauss(mu, sigma))
    try:
        log.debug(f"Rand: {key}: A: {ans}")
    except Exception:
        pass
    return ans

def sample(key: str, seq: Sequence[T], count: int) -> List[T]:
    """Overrideable random func which returns random COUNT elements of SEQ."""
    if key in overrides:
        return cast(List[T], overrides[key])
    ans = random.sample(seq, count)
    try:
        log.debug(f"Rand: {key}: A:{ans}")
    except Exception:
        pass
    return ans


def choice(key: str, seq: Sequence[T]) -> T:
    """Overrideable random func which returns one random element of SEQ."""
    if key in overrides:
        return cast(T, overrides[key])
    # Don't use random.choice here - it uses random access, which
    # is unsupported by the dict_keys view.
    ans = random.sample(seq, 1)[0]
    try:
        log.debug(f"Rand: {key}: {p.nick for p in seq} A: {ans.nick}")
    except Exception:
        pass
    return ans


def shuffle(key: str, seq: MutableSequence[Any]) -> None:
    """Overrideable random func which does an in-place shuffle of SEQ."""
    if key in overrides:
        seq.clear()
        seq.extend(overrides[key])
        return None
    random.shuffle(seq)
    return None
